Version: 1.1.0
Type: Package
Title: Fractional Response Regressions
Package: fracreg
Authors@R: person(given = c("Sulman", "Olieko"),
                   family = "Owili",
                   role = c("aut", "cre"),
                   email = "oliekosulman@gmail.com",
                   comment = c(ORCID = "0000-0001-7401-5326"))
License: GPL-3
Description: Provides routines for the estimation and specification analysis of fractional response models. Includes univariate one-part, two-part, and double-inflated three-part fractional models. Further incorporates estimators for panel data settings and addresses unobserved heterogeneity and endogeneity via correlated random effects and control function approaches. Extends fractional methodology to multivariate data via fractional multinomial logit models and handles high-dimensional multicollinear data via fractional ridge regression. Calculates analytical partial effects across all model types and includes generalised goodness-of-functional-form (GGOFF) and Regression Equation Specification Error Test (RESET) hypothesis tests. Methods are described in Papke and Wooldridge (1996) <doi:10.1002/(SICI)1099-1255(199611)11:6%3C619::AID-JAE418%3E3.0.CO;2-1>, Papke and Wooldridge (2008) <doi:10.1016/j.jeconom.2008.05.009>, Buis (2008) <http://maartenbuis.nl/software/likelihoodFmlogit.pdf>, Ramalho, Ramalho and Murteira (2011) <doi:10.1111/j.1467-6419.2009.00602.x>, Fang and Ma (2013) <doi:10.1080/02664763.2012.758246>, Mullahy (2015) <doi:10.1515/jem-2012-0006>, Murteira and Ramalho (2016) <doi:10.1080/07474938.2013.806849>, and Rokem and Kay (2020) <doi:10.1093/gigascience/giaa133>.
Language: en-GB
Date: 2026-08-08
Packaged: 2026-08-20 07:02:02 UTC; macbookpro
NeedsCompilation: no
Repository: CRAN
URL: https://sulmanolieko.github.io/fracreg/
Depends: R (>= 3.5.0)
Imports: stats, maxLik, ggplot2, grid, MASS
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), tibble, gtsummary
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Config/testthat/edition: 3
Encoding: UTF-8
Author: Sulman Olieko Owili [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7401-5326>)
Maintainer: Sulman Olieko Owili <oliekosulman@gmail.com>
Date/Publication: 2026-08-20 13:00:02 UTC
Built: R 4.6.0; ; 2026-08-20 16:04:35 UTC; unix
