# Generated by roxygen2: do not edit by hand

S3method(broom::glance,fracreg)
S3method(broom::glance,fracreghet)
S3method(broom::glance,fracregmlogit)
S3method(broom::glance,fracregpd)
S3method(broom::glance,fracregridge)
S3method(broom::tidy,fracreg)
S3method(broom::tidy,fracreg.ggoff)
S3method(broom::tidy,fracreg.pe)
S3method(broom::tidy,fracreg.ptest)
S3method(broom::tidy,fracreg.reset)
S3method(broom::tidy,fracreghet)
S3method(broom::tidy,fracreghet.pe)
S3method(broom::tidy,fracreghet.reset)
S3method(broom::tidy,fracregmlogit)
S3method(broom::tidy,fracregmlogit.pe)
S3method(broom::tidy,fracregmlogit.wtp)
S3method(broom::tidy,fracregpd)
S3method(broom::tidy,fracregpd.pe)
S3method(broom::tidy,fracregridge)
S3method(broom::tidy,fracregridge.pe)
S3method(coef,fracreg)
S3method(coef,fracreghet)
S3method(coef,fracregmlogit)
S3method(coef,fracregpd)
S3method(coef,fracregridge)
S3method(fitted,fracreg)
S3method(fitted,fracreghet)
S3method(fitted,fracregmlogit)
S3method(fitted,fracregpd)
S3method(fitted,fracregridge)
S3method(gtsummary::tbl_regression,fracreg)
S3method(logLik,fracreg)
S3method(logLik,fracreghet)
S3method(logLik,fracregmlogit)
S3method(logLik,fracregpd)
S3method(logLik,fracregridge)
S3method(nobs,fracreg)
S3method(nobs,fracreghet)
S3method(nobs,fracregmlogit)
S3method(nobs,fracregpd)
S3method(nobs,fracregridge)
S3method(plot,fracregmlogit)
S3method(plot,fracregmlogit.pe)
S3method(predict,fracreg)
S3method(predict,fracreghet)
S3method(predict,fracregmlogit)
S3method(predict,fracregpd)
S3method(predict,fracregridge)
S3method(print,fracreg)
S3method(print,fracreg.ggoff)
S3method(print,fracreg.pe)
S3method(print,fracreg.ptest)
S3method(print,fracreg.reset)
S3method(print,fracreghet)
S3method(print,fracreghet.pe)
S3method(print,fracreghet.reset)
S3method(print,fracregmlogit)
S3method(print,fracregmlogit.pe)
S3method(print,fracregmlogit.wtp)
S3method(print,fracregpd)
S3method(print,fracregridge)
S3method(print,fracregridge.pe)
S3method(print,summary.fracreg)
S3method(print,summary.fracreg.ggoff)
S3method(print,summary.fracreg.pe)
S3method(print,summary.fracreg.ptest)
S3method(print,summary.fracreg.reset)
S3method(print,summary.fracreghet)
S3method(print,summary.fracreghet.pe)
S3method(print,summary.fracreghet.reset)
S3method(print,summary.fracregmlogit)
S3method(print,summary.fracregmlogit.pe)
S3method(print,summary.fracregmlogit.wtp)
S3method(print,summary.fracregpd)
S3method(print,summary.fracregridge)
S3method(print,summary.fracregridge.pe)
S3method(residuals,fracreg)
S3method(residuals,fracreghet)
S3method(residuals,fracregmlogit)
S3method(residuals,fracregpd)
S3method(residuals,fracregridge)
S3method(stats::family,fracreg)
S3method(stats::family,fracreghet)
S3method(stats::family,fracregmlogit)
S3method(stats::family,fracregpd)
S3method(stats::family,fracregridge)
S3method(stats::formula,fracreg)
S3method(stats::formula,fracreghet)
S3method(stats::formula,fracregmlogit)
S3method(stats::formula,fracregpd)
S3method(stats::formula,fracregridge)
S3method(stats::model.frame,fracreg)
S3method(stats::model.frame,fracreghet)
S3method(stats::model.frame,fracregmlogit)
S3method(stats::model.frame,fracregpd)
S3method(stats::model.frame,fracregridge)
S3method(stats::model.matrix,fracreg)
S3method(stats::model.matrix,fracreghet)
S3method(stats::model.matrix,fracregmlogit)
S3method(stats::model.matrix,fracregpd)
S3method(stats::model.matrix,fracregridge)
S3method(stats::terms,fracreg)
S3method(stats::terms,fracreghet)
S3method(stats::terms,fracregmlogit)
S3method(stats::terms,fracregpd)
S3method(stats::terms,fracregridge)
S3method(summary,fracreg)
S3method(summary,fracreg.ggoff)
S3method(summary,fracreg.pe)
S3method(summary,fracreg.ptest)
S3method(summary,fracreg.reset)
S3method(summary,fracreghet)
S3method(summary,fracreghet.pe)
S3method(summary,fracreghet.reset)
S3method(summary,fracregmlogit)
S3method(summary,fracregmlogit.pe)
S3method(summary,fracregmlogit.wtp)
S3method(summary,fracregpd)
S3method(summary,fracregridge)
S3method(summary,fracregridge.pe)
S3method(vcov,fracreg)
S3method(vcov,fracreghet)
S3method(vcov,fracregmlogit)
S3method(vcov,fracregpd)
S3method(vcov,fracregridge)
export(fracreg)
export(fracreg.ggoff)
export(fracreg.pe)
export(fracreg.ptest)
export(fracreg.reset)
export(fracreghet)
export(fracreghet.pe)
export(fracreghet.reset)
export(fracregmlogit)
export(fracregmlogit.pe)
export(fracregpd)
export(fracregpd.pe)
export(fracregridge)
export(fracregridge.pe)
export(wtp)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,scale_color_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(graphics,par)
importFrom(grid,grid.layout)
importFrom(grid,pushViewport)
importFrom(grid,viewport)
importFrom(maxLik,maxLik)
importFrom(stats,Gamma)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,dcauchy)
importFrom(stats,dlogis)
importFrom(stats,dnorm)
importFrom(stats,fitted)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,nlminb)
importFrom(stats,nobs)
importFrom(stats,pcauchy)
importFrom(stats,pchisq)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,printCoefmat)
importFrom(stats,pt)
importFrom(stats,qcauchy)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,quasibinomial)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(stats,vcov)
