% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregridge.extractors.R
\name{nobs.fracregridge}
\alias{nobs.fracregridge}
\title{Extract the Number of Observations for fracregridge}
\usage{
\method{nobs}{fracregridge}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregridge}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An integer denoting the number of observations.
}
\description{
Extracts the number of observations used to estimate a \code{fracregridge} model.
}
