% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregridge.extractors.R
\name{fitted.fracregridge}
\alias{fitted.fracregridge}
\title{Extract Fitted Values for fracregridge}
\usage{
\method{fitted}{fracregridge}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregridge}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A matrix or array of fitted values.
}
\description{
Extracts the fitted conditional mean values from a \code{fracregridge} model.
}
