% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracreg.extractors.R
\name{logLik.fracreg}
\alias{logLik.fracreg}
\title{Extract Log-Likelihood for fracreg}
\usage{
\method{logLik}{fracreg}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracreg}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An object of class \code{logLik}.
}
\description{
Extracts the log-pseudolikelihood or log-likelihood from a fitted \code{fracreg} model.
}
