% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregpd.extractors.R
\name{fitted.fracregpd}
\alias{fitted.fracregpd}
\title{Extract Fitted Values for fracregpd}
\usage{
\method{fitted}{fracregpd}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregpd}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A numeric vector of fitted values.
}
\description{
Extracts the fitted conditional mean values from a \code{fracregpd} model.
}
