% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregmlogit.extractors.R
\name{vcov.fracregmlogit}
\alias{vcov.fracregmlogit}
\title{Extract Covariance Matrix for fracregmlogit}
\usage{
\method{vcov}{fracregmlogit}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregmlogit}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A list of covariance matrices for each choice equation.
}
\description{
Extracts the estimated variance-covariance matrices of the parameters.
}
