% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{summary.fracregmlogit}
\alias{summary.fracregmlogit}
\alias{summary.fracregmlogit.pe}
\alias{summary.fracregmlogit.wtp}
\title{Generate Summary Tables for fracregmlogit Objects}
\usage{
\method{summary}{fracregmlogit}(object, ...)
}
\arguments{
\item{object}{an object with class "fracregmlogit", "fracregmlogit.pe", or "fracregmlogit.wtp".}

\item{...}{Additional arguments passed to the printCoefmat function.}
}
\value{
Returns the object invisibly.
}
\description{
Generate tables of coefficient estimates, partial effects, and willingness to pay from
fracregmlogit-type objects.
}
\details{
This module provides summary methods for three fracregmlogit objects: \code{fracregmlogit}, \code{fracregmlogit.pe}
, and \code{fracregmlogit.wtp}. 

For \code{fracregmlogit} objects, the summary prints the number of observations, log pseudo-likelihood,
baseline choice, and the coefficient estimates with standard errors, z-statistics, and p-values
for each choice equation. 

For \code{fracregmlogit.pe} objects, it displays the marginal or discrete effects 
along with their computed standard errors (if Krinsky-Robb sampling was performed) for each choice.

For \code{fracregmlogit.wtp} objects, it provides a table of the aggregated willingness to pay 
along with its standard errors and test statistics.
}
\examples{
data("fracreg_spending")
X = fracreg_spending[,2:5]
y = fracreg_spending[,6:11]

# generate fracregmlogit summary
results1 = fracregmlogit(y, X)
summary(results1)

# generate marginal effects summary
effects1 = fracregmlogit.pe(results1, effect="marginal", se=FALSE)
summary(effects1)

}
\seealso{
\code{\link{fracregmlogit}}, \code{\link{fracregmlogit.pe}}
}
