% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregridge.extractors.R
\name{logLik.fracregridge}
\alias{logLik.fracregridge}
\title{Extract Log-Likelihood for fracregridge}
\usage{
\method{logLik}{fracregridge}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregridge}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An object of class \code{logLik}.
}
\description{
Extracts the log-likelihood from a fitted \code{fracregridge} model.
}
