% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregmlogit.extractors.R
\name{logLik.fracregmlogit}
\alias{logLik.fracregmlogit}
\title{Extract Log-Likelihood for fracregmlogit}
\usage{
\method{logLik}{fracregmlogit}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregmlogit}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An object of class \code{logLik}.
}
\description{
Extracts the log-pseudolikelihood from a fitted \code{fracregmlogit} model.
}
