% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracreg.extractors.R
\name{fitted.fracreg}
\alias{fitted.fracreg}
\title{Extract Fitted Values for fracreg}
\usage{
\method{fitted}{fracreg}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracreg}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A numeric vector of fitted values.
}
\description{
Extracts the fitted conditional mean values from a \code{fracreg} model.
}
