% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregridge.extractors.R
\name{residuals.fracregridge}
\alias{residuals.fracregridge}
\title{Extract Model Residuals for fracregridge}
\usage{
\method{residuals}{fracregridge}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregridge}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A matrix or array of residuals.
}
\description{
Extracts the response residuals from a fitted \code{fracregridge} model.
}
