% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fracregridge.extractors.R
\name{predict.fracregridge}
\alias{predict.fracregridge}
\title{Predict Method for fracregridge}
\usage{
\method{predict}{fracregridge}(object, newdata = NULL, ...)
}
\arguments{
\item{object}{A fitted model object of class \code{fracregridge}.}

\item{newdata}{An optional data frame or matrix in which to look for variables with which to predict. If omitted, the fitted values are used.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A matrix or array of predicted values.
}
\description{
Predicts conditional mean values from a fitted \code{fracregridge} model.
}
